// This is a generated file, modify: generate/templates/class_content.cc

#include <nan.h>
#include <string.h>

extern "C" {
#include <git2.h>
}

#include "../include/lock_master.h"
#include "../include/functions/copy.h"
#include "../include/indexer.h"
#include "../include/functions/sleep_for_ms.h"

#include "../include/transfer_progress.h"
#include "../include/oid.h"

#include <iostream>

using namespace std;
using namespace v8;
using namespace node;

GitIndexer::GitIndexer(git_indexer *raw, bool selfFreeing, Local<v8::Object> owner)
{
  if (!owner.IsEmpty()) {
    // if we have an owner, there are two options - either we duplicate the raw object
    // (so we own the duplicate, and can self-free it)
    // or we keep a handle on the owner so it doesn't get garbage collected
    // while this wrapper is accessible
    this->owner.Reset(owner);
    this->raw = raw;
  } else {
    this->raw = raw;
  }
  this->selfFreeing = selfFreeing;

  if (selfFreeing) {
    SelfFreeingInstanceCount++;
  } else {
    NonSelfFreeingConstructedCount++;
  }

}

GitIndexer::~GitIndexer()
{
  if (this->selfFreeing) {
    git_indexer_free(this->raw);
    SelfFreeingInstanceCount--;

    this->raw = NULL;
  }

  // this will cause an error if you have a non-self-freeing object that also needs
  // to save values. Since the object that will eventually free the object has no
  // way of knowing to free these values.
}

void GitIndexer::InitializeComponent(Local<v8::Object> target)
{
  Nan::HandleScope scope;

  Local<FunctionTemplate> tpl = Nan::New<FunctionTemplate>(JSNewFunction);

  tpl->InstanceTemplate()->SetInternalFieldCount(1);
  tpl->SetClassName(Nan::New("Indexer").ToLocalChecked());

  Nan::SetPrototypeMethod(tpl, "commit", Commit);
  Nan::SetPrototypeMethod(tpl, "free", Free);
  Nan::SetPrototypeMethod(tpl, "hash", Hash);

  Nan::SetMethod(tpl, "getSelfFreeingInstanceCount", GetSelfFreeingInstanceCount);
  Nan::SetMethod(tpl, "getNonSelfFreeingConstructedCount", GetNonSelfFreeingConstructedCount);

  Local<Function> _constructor_template = Nan::GetFunction(tpl).ToLocalChecked();
  constructor_template.Reset(_constructor_template);
  Nan::Set(target, Nan::New("Indexer").ToLocalChecked(), _constructor_template);
}

NAN_METHOD(GitIndexer::JSNewFunction)
{

  if (info.Length() == 0 || !info[0]->IsExternal()) {
    return Nan::ThrowError("A new GitIndexer cannot be instantiated.");
  }

  GitIndexer* object = new GitIndexer(static_cast<git_indexer *>(
                                        Local<External>::Cast(info[0])->Value()),
                                      Nan::To<bool>(info[1]).FromJust(),
                                      info.Length() >= 3 && !info[2].IsEmpty() && info[2]->IsObject() ? info[2]->ToObject() : Local<v8::Object>()
                                     );
  object->Wrap(info.This());

  info.GetReturnValue().Set(info.This());
}

Local<v8::Value> GitIndexer::New(const git_indexer *raw, bool selfFreeing, Local<v8::Object> owner)
{
  Nan::EscapableHandleScope scope;
  Local<v8::Value> argv[3] = { Nan::New<External>((void *)raw), Nan::New(selfFreeing), owner };
  return scope.Escape(
           Nan::NewInstance(
             Nan::New(GitIndexer::constructor_template),
             owner.IsEmpty() ? 2 : 3, // passing an empty handle as part of the arguments causes a crash
             argv
           ).ToLocalChecked());
}

NAN_METHOD(GitIndexer::GetSelfFreeingInstanceCount)
{
  info.GetReturnValue().Set(SelfFreeingInstanceCount);
}

NAN_METHOD(GitIndexer::GetNonSelfFreeingConstructedCount)
{
  info.GetReturnValue().Set(NonSelfFreeingConstructedCount);
}

git_indexer *GitIndexer::GetValue()
{
  return this->raw;
}

void GitIndexer::ClearValue()
{
  this->raw = NULL;
}


/*
   * @param TransferProgress stats
     * @return Number  result    */
NAN_METHOD(GitIndexer::Commit)
{
  Nan::EscapableHandleScope scope;

  if (info.Length() == 0 || !info[0]->IsObject()) {
    return Nan::ThrowError("TransferProgress stats is required.");
  }

// start convert_from_v8 block
  git_transfer_progress * from_stats;
  from_stats = Nan::ObjectWrap::Unwrap<GitTransferProgress>(info[0]->ToObject())->GetValue();
// end convert_from_v8 block

  giterr_clear();

  {
    LockMaster lockMaster(/*asyncAction: */false        ,    Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
                                           ,    from_stats
                         );

    int result = git_indexer_commit(
                   Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
                   ,  from_stats
                 );


    Local<v8::Value> to;
// start convert_to_v8 block
    to = Nan::New<Number>( result);
    // end convert_to_v8 block
    return info.GetReturnValue().Set(scope.Escape(to));
  }
}

/*
     */
NAN_METHOD(GitIndexer::Free)
{
  Nan::EscapableHandleScope scope;

  if (Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue() != NULL) {

    giterr_clear();

    {
      LockMaster lockMaster(/*asyncAction: */false        ,    Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
                           );

      git_indexer_free(
        Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
      );

      Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->ClearValue();
    }
    return info.GetReturnValue().Set(scope.Escape(Nan::Undefined()));
  }
}

/*
     * @return Oid  result    */
NAN_METHOD(GitIndexer::Hash)
{
  Nan::EscapableHandleScope scope;


  giterr_clear();

  {
    LockMaster lockMaster(/*asyncAction: */false        ,    Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
                         );

    const git_oid * result = git_indexer_hash(
                               Nan::ObjectWrap::Unwrap<GitIndexer>(info.This())->GetValue()
                             );

    // null checks on pointers
    if (!result) {
      return info.GetReturnValue().Set(scope.Escape(Nan::Undefined()));
    }

    Local<v8::Value> to;
// start convert_to_v8 block

    if (result != NULL) {
      // GitOid result
      to = GitOid::New(result, false  );
    } else {
      to = Nan::Null();
    }

// end convert_to_v8 block
    return info.GetReturnValue().Set(scope.Escape(to));
  }
}
Nan::Persistent<Function> GitIndexer::constructor_template;

int GitIndexer::SelfFreeingInstanceCount;
int GitIndexer::NonSelfFreeingConstructedCount;
